@@ -13,6 +13,7 @@ tests_init \
 	tilde_quoting \
 	paren_quoting \
 	multiline_field \
+	multiline_bogus_header \
 	escaped_backslash \
 	flag_order_1 \
 	flag_order_2 \
@@ -29,7 +30,8 @@ tests_init \
 	fragment_quoting_5 \
 	fragment_quoting_7 \
 	msvc_fragment_quoting \
-	msvc_fragment_render_cflags
+	msvc_fragment_render_cflags \
+	tuple_dequote
 
 comments_body()
 {
@@ -116,6 +118,14 @@ multiline_field_body()
 		pkgconf --list-all
 }
 
+multiline_bogus_header_body()
+{
+	export PKG_CONFIG_PATH="${selfdir}/lib1"
+	atf_check \
+		-s eq:0 \
+		pkgconf --exists multiline-bogus
+}
+
 escaped_backslash_body()
 {
 	atf_check \
@@ -266,3 +276,10 @@ msvc_fragment_render_cflags_body()
 		-o inline:'/I/test/include/foo /DFOO_STATIC \n' \
 		pkgconf --cflags --static --msvc-syntax foo
 }
+
+tuple_dequote_body()
+{
+	atf_check \
+		-o inline:'-L/test/lib -lfoo \n' \
+		pkgconf --with-path="${selfdir}/lib1" --libs tuple-quoting
+}
