@@ -30,7 +30,7 @@ Msys|CYGWIN*) PATH_SEP=";";;
 Haiku) LIBRARY_PATH_ENV="BELIBRARIES";;
 esac
 
-prefix="/usr/local"
+prefix="/usr"
 exec_prefix="${prefix}"
 datarootdir="${prefix}/share"
 pcpath="${exec_prefix}/lib/pkgconfig:${datarootdir}/pkgconfig"
